package dev.fitko.fitconnect.client.subscriber;

import dev.fitko.fitconnect.api.domain.model.event.Event;
import dev.fitko.fitconnect.api.domain.model.event.EventPayload;
import dev.fitko.fitconnect.api.domain.model.event.problems.Problem;
import dev.fitko.fitconnect.api.domain.model.metadata.Metadata;
import dev.fitko.fitconnect.api.domain.model.submission.Submission;
import dev.fitko.fitconnect.api.exceptions.client.FitConnectSubscriberException;
import dev.fitko.fitconnect.api.services.Subscriber;
import dev.fitko.fitconnect.client.sender.model.Attachment;
import dev.fitko.fitconnect.client.subscriber.model.ReceivedData;

import java.net.URI;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import static dev.fitko.fitconnect.api.domain.model.event.EventPayload.*;

public class ReceivedSubmission {

    private final transient Subscriber subscriber;
    private final Submission submission;
    private final Metadata metadata;
    private final ReceivedData receivedData;
    private final List<Attachment> attachments;
    private final Map<UUID, String> encryptedAttachments;

    public ReceivedSubmission(final Subscriber subscriber, final Submission submission, final Metadata metadata, final ReceivedData receivedData, final List<Attachment> attachments, final Map<UUID, String> encryptedAttachments) {
        this.subscriber = subscriber;
        this.submission = submission;
        this.metadata = metadata;
        this.receivedData = receivedData;
        this.attachments = attachments;
        this.encryptedAttachments = encryptedAttachments;

    }

    /**
     * Send an accept-submission {@link Event} to confirm that the handed in submission was accepted.
     *
     * @param problems optional {@link Problem}s
     * @see <a href="https://docs.fitko.de/fit-connect/docs/getting-started/event-log/events">FIT-Connect Events</a>
     */
    public void acceptSubmission(final Problem... problems) {
        try {
            subscriber.acceptSubmission(getEventPayload(problems));
        } catch (final Exception e) {
            throw new FitConnectSubscriberException("Accepting submission failed", e.getCause() != null ? e.getCause() : e);
        }
    }

    /**
     * Send a reject-submission {@link Event} to confirm that the handed in submission was rejected.
     *
     * @param rejectionProblems list of {@link Problem}s that give more detailed information on why the submission was rejected
     * @see <a href="https://docs.fitko.de/fit-connect/docs/getting-started/event-log/events">FIT-Connect Events</a>
     */
    public void rejectSubmission(final List<Problem> rejectionProblems) {
        try {
            subscriber.rejectSubmission(forRejectEvent(submission, rejectionProblems));
        } catch (final Exception e) {
            throw new FitConnectSubscriberException("Rejecting submission failed", e.getCause() != null ? e.getCause() : e);
        }
    }

    /**
     * Access decrypted data.
     *
     * @return data as string
     */
    public String getDataAsString() {
        return receivedData.getData();
    }

    /**
     * Get data's mime-type.
     *
     * @return mimetype string
     */
    public String getDataMimeType() {
        return receivedData.getMimeType().value();
    }

    /**
     * Get the data's schema reference.
     *
     * @return data schema as URI
     */
    public URI getDataSchemaUri() {
        return metadata.getContentStructure().getData().getSubmissionSchema().getSchemaUri();
    }

    /**
     * Access list of decrypted attachments.
     *
     * @return list of {@link Attachment}
     */
    public List<Attachment> getAttachments() {
        return attachments;
    }

    /**
     * Get the submissions {@link Metadata}.
     *
     * @return metadata
     */
    public Metadata getMetadata() {
        return metadata;
    }

    /**
     * Get the submissions caseId.
     *
     * @return caseId as UUID
     */
    public UUID getCaseId() {
        return submission.getCaseId();
    }

    /**
     * Get the submissions destinationId.
     *
     * @return destinationId as UUID
     */
    public UUID getDestinationId() {
        return submission.getDestinationId();
    }

    /**
     * Get the submissions submissionId.
     *
     * @return submissionId as UUID
     */
    public UUID getSubmissionId() {
        return submission.getSubmissionId();
    }

    private EventPayload getEventPayload(final Problem[] problems) {
        return attachments.isEmpty() ? forAcceptEvent(submission, problems) : forAcceptEventWithAttachments(submission, encryptedAttachments, problems);
    }

}
